from ptsandbox.models.ui.common import FilterValues


class SandboxArtifactsFilterValuesResponse(FilterValues):
    """
    Possible values for filters based on sources and validation results

    `<URL>/api/ui/v2/tasks/filter-values`
    """

    ...
